"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.configSet = configSet;
exports.settingShouldFallBackToNpm = settingShouldFallBackToNpm;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const util_1 = __importDefault(require("util"));
const config_1 = require("@pnpm/config");
const run_npm_1 = require("@pnpm/run-npm");
const workspace_manifest_writer_1 = require("@pnpm/workspace.manifest-writer");
const camelcase_1 = __importDefault(require("camelcase"));
const lodash_kebabcase_1 = __importDefault(require("lodash.kebabcase"));
const read_ini_file_1 = require("read-ini-file");
const write_ini_file_1 = require("write-ini-file");
async function configSet(opts, key, value) {
    if (opts.global && settingShouldFallBackToNpm(key)) {
        const _runNpm = run_npm_1.runNpm.bind(null, opts.npmPath);
        if (value == null) {
            _runNpm(['config', 'delete', key]);
        }
        else {
            _runNpm(['config', 'set', `${key}=${value}`]);
        }
        return;
    }
    if (opts.global === true || fs_1.default.existsSync(path_1.default.join(opts.dir, '.npmrc'))) {
        const configPath = opts.global ? path_1.default.join(opts.configDir, 'rc') : path_1.default.join(opts.dir, '.npmrc');
        const settings = await safeReadIniFile(configPath);
        key = (0, lodash_kebabcase_1.default)(key);
        if (value == null) {
            if (settings[key] == null)
                return;
            delete settings[key];
        }
        else {
            settings[key] = value;
        }
        await (0, write_ini_file_1.writeIniFile)(configPath, settings);
        return;
    }
    key = (0, camelcase_1.default)(key);
    await (0, workspace_manifest_writer_1.updateWorkspaceManifest)(opts.workspaceDir ?? opts.dir, {
        [key]: castField(value, (0, lodash_kebabcase_1.default)(key)),
    });
}
function castField(value, key) {
    if (typeof value !== 'string') {
        return value;
    }
    const type = config_1.types[key];
    const typeList = Array.isArray(type) ? type : [type];
    const isNumber = typeList.includes(Number);
    value = value.trim();
    switch (value) {
        case 'true': {
            return true;
        }
        case 'false': {
            return false;
        }
        case 'null': {
            return null;
        }
        case 'undefined': {
            return undefined;
        }
    }
    if (isNumber && !isNaN(value)) {
        value = Number(value);
    }
    return value;
}
function settingShouldFallBackToNpm(key) {
    return (['registry', '_auth', '_authToken', 'username', '_password'].includes(key) ||
        key[0] === '@' ||
        key.startsWith('//'));
}
async function safeReadIniFile(configPath) {
    try {
        return await (0, read_ini_file_1.readIniFile)(configPath);
    }
    catch (err) {
        if (util_1.default.types.isNativeError(err) && 'code' in err && err.code === 'ENOENT')
            return {};
        throw err;
    }
}
//# sourceMappingURL=configSet.js.map